
/**************************************************************************
**  (c) Copyright 2002, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : gcm_input.cpp
**      SYSTEM NAME    : Gnucomo - Gnu Computer Monitoring
**      VERSION NUMBER : $Revision: 1.1 $
**
**  DESCRIPTION      :  Application to store client messages into the database
**                      The client message contains a log file from one of the
**                      system logs. Gcm_input parses the log file and enters
**                      the raw data into the 'log' table of the gnucomo database.
**
**                      The system log may arrive in one of several forms:
**                      1. Log file (archived or not) from a client machine.
**                      2. Log file from a client system, arriving in a clear Email.
**                      3. Log file from a client machine, arriving in an
**                         encrypted Email.
**
**                      additional information we need that may not be availble in
**                      the content of the log is:
**                      - FQDN of the client.
**                      - Time of arrival of the log
**                      - Service that created the log.
**
**                      If the log arrives in an Email, these items probably are
**                      available in the mail header. Otherwise, they have to be
**                      provided as command line arguments.
**
**                      Command line arguments:
**                      -c <name>        Configuration name (default = gnucomo).
**                      -d <date>        Date and time of log arrival.
**                      -h <hostname>    FQDN of the client.
**                      -s <service>     Service that created the log.
**                      -T               Test mode. Do not alter the database.
**                      -v               Verbose (debug) output.
**                      -V               Print version and exit.
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Aug 29, 2002
**      LAST UPDATE     : Sep 30, 2002
**      MODIFICATIONS   : 
**************************************************************************/

/*****************************
   $Log: gcm_input.cpp,v $
   Revision 1.1  2002/10/05 10:25:49  arjen
   Creation of gcm_input and a first approach to a web interface

*****************************/

static const char *RCSID = "$Id: gcm_input.cpp,v 1.1 2002/10/05 10:25:49 arjen Exp $";

#include <getopt.h>

#include "message.h"

bool verbose = false;
bool testmode = false;

static char *Version = "gcm_input version 0.0.3 - Sep 30, 2002";


/*=========================================================================
**  NAME           : main
**  SYNOPSIS       : int main(int argc, char *argv[])
**  PARAMETERS     : 
**  RETURN VALUE   : 
**
**  DESCRIPTION    : Parse command line arguments and establish a connection
**                   to the database.
**                   When we have a database connection, parse the log file
**                   from stdin.
**
**  VARS USED      :
**  VARS CHANGED   :
**  FUNCTIONS USED :
**  SEE ALSO       :
**  LAST MODIFIED  : Sep 30, 2002
**=========================================================================
*/

int main(int argc, char *argv[])
{
   const char *usage = "Usage: gcm_input [-c configname] [-h hostname] [-d date]"
                       " [-s service] [-T] [-v] [-V]\n";

   gnucomo_config    cfg;
   char             *config_name = "gnucomo";

   /*   Parse command line arguments */

   UTC    arrival = Now();
   String  hostname(""), service("");
   int     option;

 
   while ((option = getopt(argc, argv, "c:h:d:s:TvV")) != -1)
   {
      switch (option)
      {
      case 'c':
         config_name = optarg;
         break;

      case 'h':
         hostname = optarg;
         break;

      case 'd':
         arrival = String(optarg);
         if (!arrival.proper())
         {
            cerr << "gcm_input: Invalid date string: " << optarg << ".\n";
            exit(1);
         }
         break;

      case 's':
         service = optarg;
         break;

      case 'T':
         testmode = true;
         break;

      case 'v':
         verbose = true;
         break;

      case 'V':
         cout << Version << "\n";
         exit(0);

      case '?':
      case ':':
         cerr << usage;
         exit(1);
      }
   }

   if (verbose)
   {
      cout << "Hostname = " << hostname;
      cout << " Arrival = " << arrival;
      cout << " Service = " << service << "\n";
   }

   /*  Get the configuration file */

   if (!cfg.read(config_name))
   {
      cerr << "Can not read Gnucomo configuration file for " << config_name << ".\n";
      exit(1);
   }

   if (verbose)
   {
      cout << "Config OK.\n";
   }

   /*  Try to connect to the database */

   gnucomo_database db(&cfg);

   client_message  msg(&cin, db);

   if (msg.classify(hostname, arrival, service) > 0.9)
   {
      msg.enter();
   }

   return 0;
}

